_ENV =StandardAI


--
-- Ok, so the core AI design continues to evolve.  Here's an
-- attempt to summarize the current model:
--
-- I try to avoid any AI data that persists from turn to turn. 
-- That's because making sure such data is consistent,
-- particularly across different versions of the game, is hard.
-- 
-- Thus, the AI tends to regenerate as much planning data as it
-- can from scratch each time it needs to make any decisions.
--
-- The two biggest parts of the AI's logic are those concerned
-- with how to route fleets (defined in handle_fleets.lua) and
-- how to choose planet production (defined in
-- handle_production.lua).  The two processes do share some
-- data (for example, the `threatened` table defined in
-- init_navy_plans.lua); but, they're largely independent of
-- one another.
--
-- When it is helpful to have planning data persist from turn
-- to turn, the place to put it is empire.ai_state.  Any data
-- inside empire.ai_state should be treated as strictly
-- optional.  The ai_state table itself can always be assumed
-- to exist, but, you should take care to handle the case of
-- non-existence for any subfields.
--
-- All AI planning functions are executed using a temporary AI
-- state scope.  You can see AIContext.lua to read about
-- exactly how this is implemented, but, at heart, what it
-- means is that any global variables you define inside an AI
-- script will be implicitly destroyed when that AI has 
-- finished handling the current callback function.
--
-- Along with the core planning hooks, an AI is also required
-- to implement a couple callback functions -- these are used
-- to determine how it responds to potential attacks.  The
-- callback hooks happen at a different time, and in the
-- context of a different game state, than the end turn
-- planning logic.  however, they are also called using a
-- temporary ai state scope -- which means that if they start
-- by calling "high_level_planning()", they can regenerate
-- their own versions of the same core data structures.



--
-- do we want to respond to the given defensive combat by
-- counter attacking?
--
function counter_attack(r)
	return false
end

local do_end_turn

do

	local _ENV = AIContext

  high_level_planning = timed_function('high_level_planning',|) function ()
    -- ai_debug_print | v2s | empire.ai_state

		fleet_travel_time = weak_memoize2(_G.fleet_travel_time)
    enemy_set= get_enemies(empire!)

    fleet_can_reach_star = weak_memoizeNN | function(fleet,star)
      return fleet:can_reach(star.x,star.y)
    end

    standby_colony_ships = 1

    commited_colony_value_bonus=10000

    production_desires = {}

    init_standard_planning_data()


    --ai_debug_print( v2s | potential_colonies)


    typical_colony_escort = total_power / nkeys(empire.planets)

    --ai_debug_print (..uncommitted_power,..total_power)
    --ai_debug_print(..total_colony_ships,..desired_colony_ships)

  end



  function do_end_turn_actions()
    if empire.monster_economy
      return
    end

    empire=order_player()

    --check_fleet_consistency(empire)

    --redesign_ships(empire)

    high_level_planning()

		handle_fleets()
    handle_production()

    handle_pop_transfer()

    commit_moves()
--      print('enemies', t2s | enemy_set)

    empire.ai_state.plan_list = keys(all_plans)
  --  empire.ai_state.debugging = {}
--    empire.ai_state.debugging.attack_priorities = copy | attack_priorities

  --print(t2s | empire.ai_state.plan_list)

    --check_fleet_consistency(empire)
  end

end

local SCOPE_call in _ENV!

function ai_context_call(f,...)
 	return SCOPE_call("temp AI State", {empire=order_player() }, 
    "calling AI", _ENV, f, ... )
end



function on_handle_reports()
  return ai_context_call(handle_reports,order_player())
end



end_turn_actions = timed_function('end_turn_actions',|) function ()
  --order.ai_resource_cheats(order_player())
	return ai_context_call( execute_order, ..do_end_turn_actions )
end

--print (at_war(gui_player(), all_empires[3]._ref))

if  lua_load_complete and gui_player()

    --print(v2s | routing)
    execute_order('routing', function()

      init_timing_context()

      end_turn_actions()

      --print(v2s | timing_context.report() )

    end, empire)

end

