
-- we can't actually get the design key from source_design
-- so instead concat the design_name with the ship_drawer and 
-- use that to identify ship variants
function design_id_string(design)
  return design.ship_drawer .. design.design_name
end


local function average_weapon_damage(parent_type, empire)
  local winfo = WEAPONS[parent_type]
  if winfo.is_fighter
    local ftr_weapon,alt_weapon = get_fighter_weapon(parent_type, empire)
    if alt_weapon
      return max(WEAPONS[ftr_weapon].damage_fun.average, WEAPONS[alt_weapon].damage_fun.average)
    end
    return WEAPONS[ftr_weapon].damage_fun.average
  end
  return winfo.damage_fun.average
end

local function average_tracker_evasion(speed)
  --based on the hit_chance() calculation in beam_attack_rolls.lua
  local evasion = clamp(2*speed, 0, 100)/100
  local evasion_pd = clamp(2*speed - 50, 0, 100)/100
  return (evasion + evasion_pd)/2
end

local electropulse_types = set("Electrocyte","Pulse Generator")
local average_attenuation = (1.0 + 0.75 + 0.25)/3 --beam_attack_rolls.lua
local function design_attack_value(design)
  local score = 0

  for i,mount,parent_type,mods in design.mounts()
    local part_info = PartInfo[parent_type]
    if part_info?.is_weapon
      local damage = average_weapon_damage(parent_type, design!.empire)
      if part_info.is_beam
        damage *= average_attenuation
      end

      if part_info.is_launcher
        --trackers with lower speed are more easily destroyed by PD
        local speed = part_info.speed or (part_info.is_fighter and 26 or 16) --launch_fighters.lua, launch_missiles.lua
        damage *= ( 0.1 + 0.9*average_tracker_evasion(speed) )
      end

      --print(parent_type, damage, design.implied_weapon_n(i,mods,mount,parent_type))
      score += damage * design.implied_weapon_n(i,mods,mount,parent_type)
    end

    if electropulse_types[parent_type]
      score += 3*3.5*15 * 3
    end
  end

  return score
end

local function estimated_boarding_crew_damage(hull_specs, empire)
  local crew_hitchance = empire.tactics.boarding_bonus or 0
  return hull_specs.crew*(50 + crew_hitchance)/100 * 0.5 * 0.5
end

local boarding_systems = set("Boarding Module", "Assault Shuttles", "Transporter")

local function is_combat_boarder(design)
  for i,mount,parent_type,mods in design.mounts()
    if boarding_systems[parent_type]
      return parent_type
    end
  end
  return nil
end

local function design_defence_value(hull_specs)
  return hull_specs.shields + hull_specs.armor + hull_specs.hull*0.7
end

local design_to_specs = strong_memoizeNN | function(design)
  return design.update_hull_specs()
end

function design_combat_value(design)
  local specs = design_to_specs(design)

  local attack_value = design_attack_value(design)
  if is_combat_boarder(design)
    attack_value += 2*estimated_boarding_crew_damage(specs, design.empire)
  end
  local defence_value = design_defence_value(specs)

  --print(design.design_name, attack_value, defence_value)

  return attack_value*(1+defence_value)
end

