_ENV = AIContext

function init_production_desire.defenses()
  update_production_desire.defenses()
end

function update_production_desire.defenses()

  if empire:is_at_war()
    local other_naval_power = enemy_naval_power(empire)
    local relative_power = (total_naval_power/other_naval_power)
    
    defenses_budget = empire:income() - default_reserve_budget*relative_power
    production_desires.defenses = clamp(8/(relative_power*relative_power), 4, 18)
  else
    defenses_budget = empire:income() - default_reserve_budget
    production_desires.defenses = 4
  end

end


local function find_best_defense_design(planet)

  local can_build_static_defense = planet.num_orbitals + 1 <= planet:max_orbitals()
  local raw_value_fun = function(design_key, design)
    if (not can_build_static_defense) and (design.fixed_defense or design.planet_info)
      return
    end

    if planet:cant_build_design(design_key)
      return
    end

    local ship_value = design_combat_value(design)
    if ship_value == 0
      return
    end

    return ship_value
  end

  local design_key = choose_ship_production(planet,|) {
    max_ship_production_turns = 10,
    upkeep_budget = defenses_budget,
    ..raw_value_fun,
  }

  if not design_key
    return
  end

  local build_time, projected_metal_income = estimate_design_build_time(design_key, planet)
  if empire.metal + build_time*projected_metal_income < 0
    --print('too expensive', planet.name, empire.designs[design_key].design_name, ..build_time, ..projected_metal_income)
    return --do something else until we get more metal
  end

  return design_key
end


local function can_build_defenses(planet)
  if not planet:cant_build_improvement 'Planetary Defenses'
    return true
  end

  local empire in planet

  return find_best_defense_design(planet)
end


function defense_need_metric(p)

  if not can_build_defenses(p)
    return
  end

  local threat, extra_enemy_fear in  threatened[p.star]!

  local threat = threat + extra_enemy_fear

  if not fully_developed(p) and (threat == 0 or p.industry_improvements == 0)
    return
  end

  local mult = 10

  return  (p.defenses and 0 or 1)*mult + p.raw_hammer_production
end


local function build_defense(planet)

--  if(not planet.defenses and )
--    return "Planetary Defenses"
--  end

  print('building defense on',planet.name)

  if not planet:cant_build_improvement 'Planetary Defenses'
    order.set_building_improvement(planet, 'Planetary Defenses')
    return true
  end

  local design_key =   find_best_defense_design(planet)

  if design_key and not planet:cant_build_design(design_key)
    print('choose defense design', empire.designs[design_key].design_name, planet.name)
    order.set_building_design(planet,design_key)

    register_new_ship_construction(empire.designs[design_key])

    return true 
  end

end



function dispatch.defenses()

  local planet = best_planet(defense_need_metric)

  ai_debug_print( 'dispatch.defenses', nkeys(free_planets), planet?.name )


  if(not planet) 
    --console("no best planet")
    return 'pass' 
  end

  
  if build_defense(planet)
    free_planets[planet]=nil

    --console("set defense",planet)
  else

    --console("couldn't set a defense",planet)
    return 'pass'
  end
   
end

