_ENV = AIContext

local function average_ship_build_discount(empire)
  local total_discount = 0
  local total_weight = 0
  for planet ; empire.planets
    if planet.productive
      local discount = planet:ship_build_discount()
      local weight = planet.raw_hammer_production
      total_discount += weight*discount
      total_weight += weight
    end
  end

  return total_weight > 0 and total_discount/total_weight or 1
end

--TODO call this only when an AI redesign occurs, or something
local desired_metal_to_hammer_ratio = weak_memoizeNN | function(empire)
  local average_ship_cost_mult = average_ship_build_discount(empire)

  local total = 0
  local count = 0
  for design_key, design ; empire.designs
    local specs = design_to_specs(design)
    local metal_cost, hammer_cost in specs

    local ratio = metal_cost/(hammer_cost * average_ship_cost_mult)
    
    total += ratio*ratio --more metal-costly ships are weighted more
    count += 1
  end

  return count>0 and sqrt( total/count ) or 0

end

function max_sustainable_hammer_production()
  return empire.total_metal_production/desired_metal_to_hammer_ratio(empire)
end


function desired_metal_income()
  local metal_to_hammer_ratio = desired_metal_to_hammer_ratio(empire)

  local desired_metals = 0
  for planet ; empire.planets 
    if planet.productive and planet.industry_improvements > 0
      desired_metals += planet.raw_hammer_production*metal_to_hammer_ratio
    end 
  end

  --print("desired metal income", empire.name, desired_metals, metal_to_hammer_ratio)
  return desired_metals
end


function init_production_desire.mining()
  update_production_desire.mining()
end

function update_production_desire.mining()
  --print(empire.name, ..desired_metal_production)

  if empire:missing_activity_tech('Mining') or empire.metal > 5*desired_metal_production
    production_desires.mining = nil
  elseif empire.metal_production + empire.metal < 0
    production_desires.mining = 8
  elseif empire.total_metal_production < desired_metal_production and empire.metal < 5*desired_metal_production
    production_desires.mining = 3
  elseif empire.metal_production < 0
    production_desires.mining = 1
  else
    production_desires.mining = nil
  end

end


function dispatch.mining()

  local planet,research,hammers

  ai_debug_print( 'dispatch.mining', nkeys(free_planets) )

  planet = get_max_key(free_planets,|) function(p)
    if not p.productive
      return
    end
    if empire.metal_production + empire.metal > 0 and not fully_developed(p)
      return
    end
    return p.metal_production/2 - p.raw_hammer_production
  end

  if planet and not planet:cant_set_activity('Mining')
    print(empire.name, "setting mining focus on planet", planet.name)
    order.set_activity(planet,'Mining')
    free_planets[planet] = nil
  else
    return 'pass'
  end

end