depends 'empire_intel'

_ENV = AIContext

function init_production_desire.ships()

  update_production_desire.ships()

end


function update_production_desire.ships()

  --print(empire.name, ..total_ship_value)
  local planet_count = nkeys(empire.planets)

  local other_naval_power, median_naval_power = enemy_naval_power(empire)
  if not empire:is_at_war()
    other_naval_power = median_naval_power
  end

  if other_naval_power > 0
    local relative_power = (total_naval_power/other_naval_power)

    naval_power_urgency = 1 - clamp(relative_power/0.75, 0.5, 1)
    new_ship_budget = empire:income() - default_reserve_budget*relative_power

    max_ship_production_turns = clamp(15*relative_power, 5, 45)
    production_desires.ships = clamp(8/(relative_power*relative_power), 3, 18)

    if total_ship_value <= 500*planet_count
      max_ship_production_turns = min(max_ship_production_turns, 20)
      production_desires.ships = max(production_desires.ships, 10)
    end

    print(empire.name, ..total_naval_power, ..other_naval_power, ..max_ship_production_turns, ..production_desires.ships)
  else
    -- no reference (probably in the early game and haven't met any other empires yet),
    -- fall back to vanilla-ish behaviour
    naval_power_urgency = 0
    new_ship_budget = empire:income() - default_reserve_budget

    if total_ship_value <= 100*planet_count and planet_count > 5
      max_ship_production_turns=10
      production_desires.ships=18
    elseif (total_ship_value <= 200*planet_count and planet_count > 3)
      max_ship_production_turns=15
      production_desires.ships=15
    elseif total_ship_value <= 500*planet_count
      max_ship_production_turns=20
      production_desires.ships=10
    else
     max_ship_production_turns=45
     production_desires.ships=3
    end
  end

  if new_ship_budget < 0
    production_desires.ships=nil
  end

end


local function raw_ship_production_value(design_key, design)
  if design.fixed_defense or design.planet_info
    return
  end

  local ship_value = design_combat_value(design)

  --if there's a need for scouts or transports, include that here
  local specs = design_to_specs(design)
  if specs.initial_tanks or 0 > 0
    --print("wants_transports", wants_transports())
    ship_value = max(ship_value, 4000*wants_transports())
    ship_value /= (1 - 0.9*wants_transports())
  elseif specs.scanner
    --print("wants_scouts", wants_scouts())
    ship_value = max(ship_value, 500*wants_scouts())
    ship_value /= (1 - 0.75*wants_scouts())
  end

  if ship_value == 0
    return --if the only option is a ship with 0 value, don't build anything
  end

  return ship_value
end


local function urgent_ship_build(planet)
  --only urgent build if we have at least one factory
  if planet.industry_improvements == 0
    return false
  end

  return naval_power_urgency > planet.empty_slots/planet.max_improvements
end

function dispatch.ships()

  local planet = best_industry(urgent_ship_build)
  print('dispatching ships', planet?.name)--, stack_trace(2,4))

  if not planet
    return 'pass'
  end

  local design_key = choose_ship_production(planet,|) {
    ..max_ship_production_turns,
    upkeep_budget = new_ship_budget,
    raw_value_fun = raw_ship_production_value,
  }

  --if empire.name == 'Teros' error('halt') end

  if not design_key
    return 'pass'
  end

  -- can we afford the selected design?
  ----[[
  local build_time, projected_metal_income = estimate_design_build_time(design_key, planet)
  if empire.metal + build_time*projected_metal_income < 0
    --print('too expensive', planet.name, empire.designs[design_key].design_name, ..build_time, ..projected_metal_income)
    return 'pass' --do something else until we get more metal
  end
  --]]

  local cant = planet:cant_build_design(design_key)

  if cant == 'construction_tier'
    --have to build shipyards first
    --local required_tier in design.config_table!
    design_key = get_best_infrastructure(planet)
    cant = planet:cant_build_design(design_key)

    if cant
      error('TODO: handle this fallback case more appropriately -- unable to build due to ', cant)
    end
  end

  if cant
    return 'pass'
  end

  order.set_building_design(planet, design_key)
  free_planets[planet]=nil

  local design = empire.designs[design_key]
  register_new_ship_construction(design)

  print('choose design', design.design_name, planet.name)

end