_ENV = AIContext

depends "design_evaluation"


function init_navy_composition()

  total_naval_power = 0

  total_ship_value = 0
  target_ship_value = target_navy_power()

  constructed_ships = {}
  constructed_ship_designs = setdefault({}, 0)

  for _,fleet ; empire.fleets
    for ship ; fleet.ships
      total_naval_power += ship_power_score | ship
      total_ship_value += combined_cost | ship
      constructed_ships[ship] = true

      --print | v2s | ship.source_design
      local design_id = design_id_string(ship.source_design)
      constructed_ship_designs[design_id] += 1
      constructed_ship_designs.total += 1

      if ship.scanner
        constructed_ship_designs.scouts += 1
      end

      if ship.carrying_tanks > 0
        constructed_ship_designs.tanks += ship.carrying_tanks
      end

    end
  end

  design_to_specs = strong_memoizeNN | function(design)
    return design.update_hull_specs()
  end

  design_to_ship = strong_memoizeNN | function(design)
    local ship= EmulateRecorded.all_ships | design.genShip()
    ship.is_fixed = ship.drawer_table.fixed_defense
    return ship
  end

end

function register_new_ship_construction(design)
  constructed_ships[design_to_ship(design)] = 'building'
  constructed_ship_designs[design_id_string(design)] += 1
  constructed_ship_designs.total += 1

  local specs = design_to_specs(design)

  if (specs.initial_tanks or 0) > 0
    constructed_ship_designs.tanks += specs.initial_tanks
  end

  if specs.scanner
    constructed_ship_designs.scouts += 1
  end

end


function get_best_infrastructure(planet)
  if planet.num_orbitals+1>planet:max_orbitals()
    -- this is a troublesome case -- the simplest fix is to just
    -- scrap a station

    local i, orbital = get_best_value(planet.orbitals, | ) function(b,a)
      return lexical_compare(
        a.construction_tier or 0, b.construction_tier or 0,
        combined_cost(a), combined_cost(b),
        a._ikey,b._ikey)
    end

    if not i
      error 'this is odd -- the planet really ought to have at least 1 orbital'
    end

    local _TRACE_INFO = planet.name

    print(..i,..orbital)
    print(..orbital.name)

    if orbital
        planet:scrap_orbital(i)
    end
  end

  local design_key, design = get_best_value(empire.designs, | ) function(d0,d1)
  local s0 = design_to_specs(d0)
  local s1 = design_to_specs(d1)

    --if planet:cant_build_design(d0!._ikey) 
  --    print(v2s | d0)
--    end

  return lexical_compare(
    planet:cant_build_design(d0!._ikey) and 0 or 1, 
    planet:cant_build_design(d1!._ikey) and 0 or 1, 

    d0.planet_info and 0 or 1, d1.planet_info and 0 or 1, 
    s0.construction_tier or 0, s1.construction_tier or 0,
    combined_cost(s0), combined_cost(s1) )
  end

  return design_key, design
end

--[[ deprecated
function build_best_infrastructure(planet)
  --errorf('build best infrastructure case at: %s',planet.name)

  local design_key = get_best_infrastructure(planet)

  if not design_key
     error 'thats odd, is the design set empty?' 
  end

  local cant = planet:cant_build_design(design_key)
  
  if cant
    print('TODO: handle this fallback case more appropriately -- unable to build due to ',cant)
    order.set_activity(planet,'Trade')
  else
    order.set_building_design(planet, design_key )
  end
  --print(..design_name)

end
--]]

-- returns the estimated # of turns to complete, as well as 
-- the projected metal income if construction is started on this design
function estimate_design_build_time(design_key, planet)

  --NOTE planet:design_cost() already includes ship build discount!
  local hammer_cost, metal_cost = planet:design_cost(design_key)

  --consider any pre-existing progress
  local hammers, building_metal = 0,0
  if planet.building_design == design_key
    hammers = planet.hammers
    building_metal = planet.building_metal
  else
    local config_name = planet.empire.designs[design_key].config_name
    for i,v ; planet.incomplete_projects?
      if v.config_name==config_name
        --can use this prior project
        hammers = v.hammers
        building_metal = v.building_metal
        break
      end
    end
  end

  hammer_cost = max(hammer_cost - hammers, 0)
  metal_cost = max(metal_cost - building_metal, 0)

  --no hammer production, will never complete
  if planet.raw_hammer_production == 0 and hammer_cost > 0
    return 10000, empire.metal_production, metal_cost
  end

  --first, consider hammers
  local turns_to_complete = max(hammer_cost - planet.overflow_hammers, 0)/planet.raw_hammer_production

  --next, consider metal
  local projected_metal_use = metal_cost/max(turns_to_complete, 1)
  local projected_metal_income = empire.metal_production + planet.metal_use - projected_metal_use

  --no metal production, will never complete
  if empire.total_metal_production == 0 and metal_cost > 0
    return 10000, projected_metal_income, metal_cost
  end

  --anticipate metal shortages
  if projected_metal_income < 0
    local turns_to_metal_stall = floor(-empire.metal/projected_metal_income)
    --print(..turns_to_complete, ..turns_to_metal_stall)
    if turns_to_complete >= turns_to_metal_stall
      local total_metal_use = 0
      for p ; empire.planets 
        if not p.no_colony and p ~= planet
          total_metal_use += p.metal_use
        end
      end

      local projected_total_metal_use = total_metal_use + projected_metal_use

      local projected_metal_cost_at_stall = metal_cost - projected_metal_use*turns_to_metal_stall
      local projected_allocated_metal_income = projected_metal_use*(empire.total_metal_production/projected_total_metal_use)
      local stalled_turns = projected_metal_cost_at_stall/projected_allocated_metal_income
      
      turns_to_complete = max(turns_to_complete, turns_to_metal_stall + stalled_turns)
      --print(..turns_to_metal_stall, ..stalled_turns, ..projected_total_metal_use, ..total_metal_production)
    end
  end

  return ceil(turns_to_complete), projected_metal_income, metal_cost
end


--criteria should have the following fields:
--  max_ship_production_turns
--  upkeep_budget
--  raw_value_fun(design_key, design)
function choose_ship_production(planet, criteria)

  -- a value between 0 and 1
  -- affects how much the AI values getting a ship sooner vs getting it later
  local patience = 0.2^(1/criteria.max_ship_production_turns)

  local design_key, value = get_max_key(empire.designs, | ) function(key, design)

    local upkeep in design_to_specs(design)
    if upkeep > criteria.upkeep_budget
      --print('too expensive', planet.name, design.design_name, ..upkeep, ..criteria.upkeep_budget)
      return
    end

    local ship_value = criteria.raw_value_fun(key, design)
    if not ship_value
      return
    end

    --apply a modifier based on the proportion of our fleet that is already of this design, to encourage variety
    local already_built = constructed_ship_designs[design_id_string(design)]/constructed_ship_designs.total
    --print(empire.name, design.design_name, "variety mult", (1-already_built)^2)
    ship_value *= (1-already_built)^2

    local build_time, projected_metal_income, metal_cost = estimate_design_build_time(key, planet)

    --do we need shipyard infrastructure to build this that we don't yet have?
    --if so, add the time required to create the infrastructure to the estimated build time
    if planet:cant_build_design(key) == 'construction_tier'
      local infrastructure = get_best_infrastructure(planet)
      build_time += estimate_design_build_time(infrastructure, planet)
      print(design.design_name, build_time)
    end

    --since static defenses have limited strategic flexibility, reduce patience for them
    local patience_mult = design.fixed_defense and 2/3 or 1
    local p = patience * patience_mult

    --adjust ship value to account for build time
    ship_value *= (p^build_time)/build_time

    --print(planet.name, empire.name, design.design_name, ..ship_value, ..build_time, ..patience, ..projected_metal_income)

    return ship_value
  end

  return design_key
end


function wants_scouts()
  if num_scoutable_stars == 0
    return 0
  end

  other_naval_power = enemy_naval_power(empire)
  local scout_mult = 1
  if total_naval_power < other_naval_power
    local power_ratio = total_naval_power/other_naval_power
    scout_mult = power_ratio*power_ratio
  end

  local scouts in constructed_ship_designs

  local desired_scouts = scout_mult * min(num_scoutable_stars/4, max(nkeys(empire.stars), 3))
  --print(empire.name, ..scouts, ..desired_scouts)

  return saturate(1 - scouts/desired_scouts)
end

function wants_transports()
  if total_naval_power == 0
    return 0
  end

  --if we're behind in naval power, build less transports
  other_naval_power = enemy_naval_power(empire)
  local power_ratio = other_naval_power > 0 and other_naval_power/total_naval_power or 1
  local power_per_tank = power_ratio*power_ratio*6000

  local target_tanks = total_naval_power / power_per_tank
  local current_tanks = constructed_ship_designs.tanks

  target_tanks = min(target_tanks,10,max(2,nkeys(empire.planets)))

  print(empire.name, ..current_tanks, ..target_tanks)

  return saturate(1 - current_tanks/target_tanks)
end